<?php

namespace Drupal\form_helper_widget\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\field_ui\FieldUI;
use Drupal\form_helper_widget\Services\FormHelperWidgetConf;

/**
 * Class FormHelperWidgetDeleteForm.
 *
 * @package Drupal\form_helper_widget\Form
 */
class FormHelperWidgetDeleteForm extends ConfirmFormBase {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'form_helper_widget.delete';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Do you want to delete the widget @name', ['@name' => $this->getValueFromRoute('widgetId')]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    $entityTypeId = $this->getValueFromRoute('entityTypeId');
    $mode = $this->getValueFromRoute('mode');
    $entityType = \Drupal::entityTypeManager()->getDefinition($entityTypeId);
    $route_parameters = FieldUI::getRouteBundleParameter($entityType, $this->getValueFromRoute('bundle'));
    $route_parameters['form_mode_name'] = $mode;

    $url = new Url("entity.entity_form_display." . $entityTypeId . '.' . $mode,
      $route_parameters);

    return $url;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    FormHelperWidgetConf::me()->deleteWidget(
      $this->getValueFromRoute('entityTypeId'),
      $this->getValueFromRoute('bundle'),
      $this->getValueFromRoute('mode'),
      $this->getValueFromRoute('widgetId')
    );

    $form_state->setRedirectUrl($this->getCancelUrl());
  }

  /**
   * {@inheritdoc}
   */
  protected function getValueFromRoute($key) {
    return \Drupal::routeMatch()->getParameter($key);
  }

}
