<?php

namespace Drupal\form_helper_widget\Services;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\Entity\ContentEntityFormInterface;
use Drupal\Core\Form\ConfirmFormInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class FormHelperWidgetManager.
 *
 * @package Drupal\form_helper_widget\Services
 */
class FormHelperWidgetManager {

  const SERVICE_NAME = 'form_helper_widget.manager';

  /**
   * Retourne le singleton.
   *
   * @return static
   *   Le singleton.
   */
  public static function me() {
    return \Drupal::service(static::SERVICE_NAME);
  }

  /**
   * Retourne le widget de traduction.
   *
   * @param array $form
   *   The initial form.
   * @param string $stringId
   *   THe title.
   * @param array $description
   *   The description.
   *
   * @return array
   *   The form.
   */
  public function getWidget(array &$form, $stringId, array $description) {

    $slug = $this->slugify($stringId);

    $form['#' . FormHelperWidgetConf::THIRD_PARTY_ID][] = [
      'slug'                         => $slug,
      FormHelperWidgetConf::FIELD_ID => $stringId,
    ];

    return [
      '#theme'   => 'form_helper_widget',
      '#title'   => $stringId,
      '#content' => nl2br($description['value']),
    ];
  }

  /**
   * Slugify a string.
   *
   * @param string $string
   *   THe string to slugify.
   *
   * @return string
   *   The slugified string.
   */
  public function slugify($string) {
    return ContainerBuilder::underscore($string);
  }

  /**
   * Add the widget in the current form.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $formState
   *   THe formstate.
   */
  public function showWidgetInForm(array &$form, FormStateInterface $formState) {
    $form_object = $formState->getFormObject();
    if ($form_object instanceof ContentEntityFormInterface && !$form_object instanceof ConfirmFormInterface) {

      $storage = $formState->getStorage();
      if (!empty($storage['form_display'])) {

        // Récupération des paramètres.
        $thirdPartyData = $storage['form_display']->getThirdPartySettings(FormHelperWidgetConf::THIRD_PARTY_ID);
        if (!empty($thirdPartyData)) {
          foreach ($thirdPartyData as $data) {
            $values = $this->getWidget($form, $data[FormHelperWidgetConf::FIELD_ID], $data[FormHelperWidgetConf::FIELD_DESCRIPTION]);
            $values['#weight'] = $data['weight'];
            $fieldId = empty($data[FormHelperWidgetConf::FIELD_FIRST_ID]) ? $data[FormHelperWidgetConf::FIELD_ID] : $data[FormHelperWidgetConf::FIELD_FIRST_ID];
            $form[$fieldId] = $values;
          }
        }
      }
    }
  }

}
